<?php
  /*
   * Plugin Name: Hierarchical Pages
   * Version: 1.0
   * Plugin URI: http://www.wlindley.com/website/hierpage
   * Description: Adds a sidebar widget to display a context-based list of "nearby" pages. Updated to Wordpress 2.8 style widget classes.
   * Author: William Lindley
   * Author URI: http://www.wlindley.com/
   */

  /*  Copyright 2009 William Lindley (email : wlindley -at- wlindley -dot- com)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */

class HierPageWidget extends WP_Widget
{
  /**
   * Declares the HierPageWidget class.
   *
   */
  function HierPageWidget(){
    $widget_ops = array('classname' => 'widget_hier_page', 'description' => __( "Hierarchical Page Directory Widget") );
    $control_ops = array('width' => 300, 'height' => 300);
    $this->WP_Widget('hierpage', __('Hierarchical Pages'), $widget_ops, $control_ops);
  }

  /**
   * Helper function
   *
   */
  function hierpages_list_pages($args = '') {
    global $post;
    global $wp_query;

    parse_str($args, $r);
    if ( !isset($r['echo']) )
      $r['echo'] = 1;
    $sort_column = $r['sort_column'];

    $output = '';

    // Query pages.  NOTE: The array is sorted in alphabetical, or menu, order.
    $pages = & get_pages($args);
    if ( $pages ) {

      $page_info = Array();

      $current_post = $wp_query->get_queried_object_id();

      foreach ( $pages as $page ) {
	$page_info[$page->ID]['parent'] = $page->post_parent;
	$page_info[$page->post_parent]['children'][] = $page->ID;
      }

      // Display the front page?
      $front_page = -1; // assume no static front page
      if ('page' == get_option('show_on_front')) {
	$front_page = get_option('page_on_front');
	// Regard flag: always show front page?  Otherwise: Show front page only if it has children
	if (($r['show_home'] == 'yes') || (sizeof($page_info[$front_page]['children']))) {
	  $page_info[$front_page]['show'] = 1;	// always show front page
	}
      }
	
      if (is_page()) {
	if ($post->ID != $front_page ) {
	  // The current page is always shown, unless it is the static front page (see above)
	  $page_info[$post->ID]['show'] = 1;
	}
	
	// show the current page's children.
	foreach ( $page_info[$current_post]['children'] as $child ) {
	  $page_info[$child]['show'] = 1;
	}

	$post_parent = $page_info[$current_post]['parent'];
	if ($r['show_siblings'] == 'yes') {
	  // if showing siblings, add the current page's parent's other children.
	  foreach ( $page_info[$post_parent]['children'] as $child ) {
	    if ($child != $front_page) {
	      $page_info[$child]['show'] = 1;
	    }
	  }

	  # wl 2007-06-22 v0.4 -- Also show parent node's siblings.
	  $post_grandparent = $page_info[$post_parent]['parent'];
	  if ($post_grandparent) {
	    foreach ( $page_info[$post_grandparent]['children'] as $child ) {
	      if ($child != $front_page) {
		$page_info[$child]['show'] = 1;
	      }
	    }
	  }
	}
	
	// add all ancestors of the current page.
	while ($post_parent) {
	  $page_info[$post_parent]['show'] = 1;
	  $post_parent = $page_info[$post_parent]['parent'];
	}
      }
      
      // add all children of the root node, but only to single depth.
      if ($r['show_root'] == 'yes') {
	foreach ( $page_info[0]['children'] as $child ) {
	  if ($child != $front_page) {
	    $page_info[$child]['show'] = 1;
	  }
	}
      }
      
      // Add pages that were selected
      $page_tree = Array();
      foreach ( $pages as $page ) {
	if (($page->menu_order > -1000) && ($page_info[$page->ID]['show'])) {
	  $my_includes .= "$page->ID,";
	}
      }
      
      //  added blank title_li for Wordpress 2.3
      $output .= wp_list_pages( "title_li=&sort_column=$sort_column&include=$my_includes");
    }

    $output = apply_filters('wp_list_pages', $output);
    
    if ( $r['echo'] )
      echo $output;
    else
      return $output;
  }

  /**
   * Displays the Widget
   *
   */
  function widget($args, $instance){
    extract($args);
    $title = apply_filters('widget_title', empty($instance['title']) ? 'Pages' : $instance['title']);
    $menu_order = $instance['menu_order'];
    $show_root = $instance['show_root'];
    $show_home = $instance['show_home'];
    $show_siblings = $instance['show_siblings'];
      
    # Before the widget
    echo $before_widget;

    # The title
    if ( $title )
      echo $before_title . $title . $after_title;

    echo "<ul>";
    $page_options = "show_siblings=$show_siblings&show_root=$show_root&show_home=$show_home";
    if ($menu_order == 'yes') {
      $page_options .= "&sort_column=menu_order,post_title";
    } else {
      $page_options .= "&sort_column=post_title";
    }

    $this->hierpages_list_pages($page_options);
    echo "</ul>";
    echo $after_widget;

  }

  /**
   * Saves the widget's settings.
   *
   */
  function update($new_instance, $old_instance){
    $instance = $old_instance;

    $instance['title'] = strip_tags(stripslashes($new_instance['title']));
    $instance['menu_order'] = strip_tags(stripslashes($new_instance['menu_order']));
    $instance['show_siblings'] = strip_tags(stripslashes($new_instance['show_siblings']));
    $instance['show_root'] = strip_tags(stripslashes($new_instance['show_root']));
    $instance['show_home'] = strip_tags(stripslashes($new_instance['show_home']));

    return $instance;
  }

  /**
   * Creates the edit form for the widget.
   *
   */
  function form($instance){
    //Defaults
    $instance = wp_parse_args( (array) $instance, array('title'=>'', 'lineOne'=>'Hello', 'lineTwo'=>'World') );

    $title = htmlspecialchars($instance['title']);
    $menu_order = htmlspecialchars($instance['menu_order']);
    $show_siblings = htmlspecialchars($instance['show_siblings']);
    $show_root = htmlspecialchars($instance['show_root']);
    $show_home = htmlspecialchars($instance['show_home']);

      # Output the options
    echo '<p style="text-align:right;"><label for="' . $this->get_field_name('title') . '">' . 
      __('Title:') . 
      ' <input style="width: 200px;" id="' . $this->get_field_id('title') . 
      '" name="' . $this->get_field_name('title') . 
      '" type="text" value="' . $title . '" /></label></p>';
    echo '<p style="text-align:right;"><label for="' . $this->get_field_name('menu_order') . '">' . 
      __('Sort in menu order?<br>(else alphabetical)') . 
      ' <input id="' . $this->get_field_id('menu_order') .
      '" name="' . $this->get_field_name('menu_order') . '"' .
      (strlen($menu_order) ? 'checked="yes" ' : '') .
      ' type="checkbox" value="yes" /></label></p>';
    echo '<p style="text-align:right;"><label for="' . $this->get_field_name('show_siblings') . '">' . 
      __('Show siblings to the current page?') .
      ' <input id="' . $this->get_field_id('show_siblings') .
      '" name="' . $this->get_field_name('show_siblings') . '"' .
      (strlen($show_siblings) ? 'checked="yes" ' : '') .
      ' type="checkbox" value="yes" /></label></p>';
    echo '<p style="text-align:right;"><label for="' . $this->get_field_name('show_root') . '">' . 
      __('Always show top-level pages?') . 
      ' <input id="' . $this->get_field_id('show_root') .
      '" name="' . $this->get_field_name('show_root') . '"' .
      (strlen($show_root) ? 'checked="yes" ' : '') .
      ' type="checkbox" value="yes" /></label></p>';
    echo '<p style="text-align:right;"><label for="' . $this->get_field_name('show_home') . '">' . 
      __('Show the static home page?<br>(always shown if it has child pages)') . 
      ' <input id="' . $this->get_field_id('show_home') .
      '" name="' . $this->get_field_name('show_home') . '"' .
      (strlen($show_home) ? 'checked="yes" ' : '') .
      ' type="checkbox" value="yes" /></label></p>';
  }

}// END class

/**
 * Register Hierarchical Pages widget.
 *
 * Calls 'widgets_init' action after the widget has been registered.
 */
function HierPageInit() {
  register_widget('HierPageWidget');
}
add_action('widgets_init', 'HierPageInit');
?>
